import json


class Output:
    """
    Base class for output handlers.
    """

    def write(self, data, file_name) -> None:
        """
        Write data to the specified output.

        :param data: Data to be written.
        :param file_name: Name of the file to be created.
        :return: None
        """
        raise NotImplementedError("Subclasses should implement this method.")


class GCSOutput(Output):
    """
    Class to handle output to Google Cloud Storage (GCS).
    """

    def __init__(self, bucket_name, blob_path):
        """
        Initialize the GcsOutput with the bucket and blob names.

        :param bucket_name: Name of the GCS bucket.
        :param blob_path: Name of the blob in the GCS bucket.
        """
        self.bucket_name = bucket_name
        self.blob_path = blob_path

    def write(self, data, file_name) -> None:
        """
        Write data to the specified GCS bucket and blob.

        :param data: Data to be written to GCS.
        :param file_name: Name of the file to be created in GCS.
        :return: None
        """
        from google.cloud import storage

        client = storage.Client()
        bucket = client.bucket(self.bucket_name)
        blob_name = f"{self.blob_path}/{file_name}.json"
        blob = bucket.blob(blob_name)  # Create a blob object with the specified name

        blob.upload_from_string(data=json.dumps(data), content_type="application/json")  # Assuming data is a string



import datetime

TIME_FORMAT = "%Y%m%d_%H%M%S"


def get_base_name(name):
    return f"{name}_{datetime.datetime.now().strftime(TIME_FORMAT)}.json"


def run_pipeline(pipeline_name, ingestion: object, outputs: list = None, name_template=get_base_name):
    """
    Function to run a pipeline with the given name, ingestion method, and outputs.

    :param name: Name of the pipeline.
    :param ingestion: Ingestion method to fetch data.
    :param outputs: List of output methods to process the data.
    :return: None
    """

    if outputs is None:
        outputs = []
    data = ingestion.get()
    print("Data fetched from ingestion:", data)

    for output in outputs:
        print(f"Processing output with {output.__class__.__name__}")
        output.write(data, name_template(pipeline_name))



from typing import Callable

import requests


class DataSource:
    """
    Base class for data sources.
    """

    def get(self) -> dict:
        """
        Fetch data from the data source.

        :return: Data fetched from the data source.
        """
        raise NotImplementedError("Subclasses should implement this method.")


class HttpDataSource(DataSource):
    def __init__(self, host: str, endpoint: str):
        self.host = host
        self.endpoint = endpoint

    def get(self):
        """
        Fetch data from the specified host and endpoint using the process function.

        :param host: The base URL of the API.
        :param endpoint: The specific endpoint to fetch data from.
        :return: Processed data from the API.
        """
        response = requests.get(f"{self.host}/{self.endpoint}")
        response.raise_for_status()
        return response.json()


class PythonDataSource(DataSource):
    """
    Class to handle Python data source.
    """

    def __init__(self, python_function: Callable[[], dict]):
        """
        Initialize the PythonDataSource with the data.

        :param data: Data to be used by the PythonDataSource.
        """
        self.python_function = python_function

    def get(self) -> dict:
        """
        Get the data from the PythonFunction.

        :return: The data stored in the PythonFunction.
        """
        return self.python_function()



import importlib.metadata

from miniduct.datasources import DataSource, HttpDataSource, PythonDataSource  # noqa: F401
from miniduct.outputs import GCSOutput, Output  # noqa: F401
from miniduct.pipeline import run_pipeline  # noqa: F401

try:
    # This will read the version from the installed package metadata
    __version__ = importlib.metadata.version("miniduct")
except importlib.metadata.PackageNotFoundError:
    # This is a fallback for when the package is not installed (e.g., running from source)
    __version__ = "0.0.0-dev"
